/*
 * Component Name: isamTokenAuthenticationLibrary
 * File Name: isamTokenAuthExterns.h
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2018
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */
#ifndef _ISAM_TOKEN_AUTH_EXTERNS_H_
#define _ISAM_TOKEN_AUTH_EXTERNS_H_

/*
 * Only for Windows compilation.
 */
#if defined _WIN32 || _WIN64
#pragma once

#ifdef ISAMTOKENAUTHENTICATIONLIBRARY_EXPORTS
#define ISAMTOKENAUTHENTICATIONLIBRARY_API __declspec(dllexport)
#else
#define ISAMTOKENAUTHENTICATIONLIBRARY_API __declspec(dllimport)
#endif

#ifndef _CALLBACKS_DEFINED_
extern ISAMTOKENAUTHENTICATIONLIBRARY_API int authenticateToken(
		const char *username,
		const char *password,
		const char *scopes,
		const char *token,
		const int actionCode,
		char **errorDescription
);
extern ISAMTOKENAUTHENTICATIONLIBRARY_API int isamInitialize(
		const char *hostname,
		const int port,
		const char *endpoint,
		const bool isSSL,
		const char *clientId
);

#else

extern ISAMTOKENAUTHENTICATIONLIBRARY_API int isamInitialize(
		const char *hostname,
		const int port,
		const char *endpoint,
		const bool isSSL,
		const char *clientId,
		int (*userCredCallback)(char **username, char **password),
		int (*passwordCallback)(const char *username, char **password));

extern ISAMTOKENAUTHENTICATIONLIBRARY_API int authenticateToken (
		const char *scopes,
		const char *token,
		char **errorDescription);

#endif

extern ISAMTOKENAUTHENTICATIONLIBRARY_API void isamFinalize();
#else
/*
 * Linux -> Anything but Windows.
 */

#ifndef _CALLBACKS_DEFINED_
/*
 * This function is called by the client to enable a user to authenticate
 * against ISAM using tokens, like an RFID tag, magnetic strip ID cards etc.
 *
 * char *username (in): OPTIONAL - Username of the user (can be NULL).
 * char *password (in): OPTIONAL - Password of the user (can be NULL).
 * char *scopes (in): OPTIONAL - <TODO::>
 * char *token (in): MANDATORY - The token string that is used to identify
 * 					a user.
 * int actionCode (in): MANDATORY - Action to be taken (One of the following:
 * _REQ_REFRESH_AUTH, _REQ_REGISTER_USER OR _REQ_REFRESH_PASSWORD.
 *
 * char **errorDescription (in): MANDATORY - Buffer to receive the error
 * 					description string.
 *
 * int (out): Status of the call.
 * 	_SUCCESS or error code.
 */
extern int authenticateToken(const char *username, const char *password,
		const char *scopes, const char *token, const int actionCode,
		char **errorDescription);

/*
 * This function is called by the client to initialize the 'C'client library.
 * This function need to be called before any other method is called.
 *
 * char *hostname (in): URL (or hostname/IP address of ISAM instance).
 * int port (in): Port number on which ISAM is listening to the incoming requests.
 * char *endpoint (in): Mobile junction configured on ISAM.
 * bool isSSL (in): Is the connection un/secured (http OR https)
 * char *clientId (in): Client Id configured for this client lib on ISAM.
 *
 * int (out): Status of the call.
 * 	SUCCESS or error code.
 */
extern int isamInitialize(const char *hostname, const int port,
		const char *endpoint, const bool isSSL, const char *clientId);

#else
/*
 * This function is called by the client to initialize the 'C'client library.
 * This function need to be called before any other method is called.
 *
 * char *hostname (in): URL (or hostname/IP address of ISAM instance).
 * int port (in): Port number on which ISAM is listening to the incoming requests.
 * char *endpoint (in): Mobile junction configured on ISAM.
 * bool isSSL (in): Is the connection un/secured (http OR https)
 * char *clientId (in): Client Id configured for this client lib on ISAM.
 * userCredCallback (in): User Cred callback function.
 * passwordCallback (in): User password callback function.
 *
 * int (out): Status of the call.
 * 	SUCCESS or error code.
 */
extern int isamInitialize(const char *hostname, const int port,
		const char *endpoint, const bool isSSL, const char *clientId,
		int (*userCredCallback)(char **username, char **password),
		int (*passwordCallback)(const char *username, char **password));

/*
 * This function is called by the client to enable a user to authenticate
 * against ISAM using tokens, like an RFID tag, magnetic strip ID cards etc.
 *
 * char *scopes (in): OPTIONAL - <TODO::>
 * char *token (in): MANDATORY - The token string that is used to identify
 * 					a user.
 * char **errorDescription (in): MANDATORY - Buffer to receive the error
 * 					description string.
 *
 * int (out): Status of the call.
 * 	_SUCCESS or error code.
 */
extern int authenticateToken(const char *scopes, const char *token,
		char **errorDescription);

#endif
/*
 * This function is called by the client finalize the 'C'client library.
 * This should be called whenever to complete the execution of the client
 * application to cleanup the resources.
 */
extern void isamFinalize();

#endif

/*
 * Generic -> For all platforms.
 */
extern const char *SUCCESSFUL_AUTH;
extern const char *NULL_TOKEN;
extern const char *INVALID_INPUT;
extern const char *USER_NOT_REGISTERED;
extern const char *MANDATORY_PARAMS_MISSING;
extern const char *USERNAME_PASSWORD_MECH_INVALID;
extern const char *GRANT_ACTIVE;
extern const char *PASSWORD_REFRESH;
extern const char *ANOTHER_CARD_ALREADY_REGISTERED;
extern const char *CARD_DISABLED;
extern const char *CARD_ALREADY_REGISTERED;
extern const char *INVALID_ARGUMENT;
extern const char *GRANT_TYPE_NOT_SET;
extern const char *GRANT_TYPE_REFRESH_TOKEN;
extern const char *UNKNOWN_ERROR;
extern const char *UNABLE_TO_GET_ERROR_DESC;
extern const char *TOKEN_LOCKED;
extern const char *CLIENT_NOT_INITIALIZED;

enum _errorCodes {
	_FAILURE = -1,
	_SUCCESS,
	_REGISTER_USER,
	_REFRESH_PASSWORD,
	_INVALID_INPUTS,
	_CARD_DISABLED,
	_USER_REGISTERED,
	_CARD_REGISTERED,
	_ACTIVE_GRANT,
	_TOKEN_LOCKED,
	_CLIENT_NOT_INITIALIZED,
	_CANNOT_CONTINUE,
	_UNKNOWN_ERROR
} errorCodes;

#ifndef _CALLBACKS_DEFINED_
enum _actionCodes {
	_REQ_REFRESH_AUTH = _SUCCESS, _REQ_REGISTER_USER, _REQ_REFRESH_PASSWORD
} actionCodes;
#endif

#ifndef MAX_PARAM_SIZE
#define  MAX_PARAM_SIZE 512
#endif

#endif /* _ISAM_TOKEN_AUTH_EXTERNS_H_ */
