# ISAM Token Authentication 'C' Library

This is a 'C' Library for OAuth to interact with ISAM. This utilizes the tokens off Magnetic Strip/RFID reader to generate long term OAuth grants for the user.

The library is exposes the following functions and macros.

<h2 id="calls">APIs exposed</h2>
<a href="#initWithCallbacks">isamInitialize with callbacks</a>
<br/>
<a href="#authWithCallbacks">authenticateToken with callbacks</a>
<br/>
<a href="#final">isamFinalize</a>
<br/>


<h2 id="calls">APIs NOT exposed</h2>
<a href="#initWithoutCallbacks">isamInitialize without callbacks</a>
</br>
<a href="#authWithoutCallbacks">authenticateToken without callbacks</a>
</br>


There are two variations of `isamInitialize` and `authenticateToken` functions.

* The first one being in which the client code receives the result (error codes) back for each call to <a href="#authWithoutCallbacks">authenticateToken</a> and based on the error code received, client needs to take an appropriate action.

* The second implementation is the way of Callback functions. During <a href="#initWithCallbacks">isamInitialize</a>, client pass 2 callback functions to the ISAM Token Authentication 'C' Library and makes the first call to <a href="#authWithCallbacks">authenticateToken</a>. After that ISAM Token Authentication 'C' Library calls the appropriate callback function, when required.

## Functions to be used with callbacks

<h2 id="initWithCallbacks">isamInitialize</h2>
This function is used to initialize the ISAM 'C' library. ISAM 'C' library has to be initialized before any other APIs are called. 2 callback functions are registered in this call so that ISAM Token Authentication 'C' Library can call these functions at the appropriate time.

### Parameters
| Name | Type | Description |
| -----| ---- | ----------- |
| hostname | const char* | The URL (hostname or IP address) of ISAM instance |
| port | const int | Port numner on which ISAM is listening for requests. |
| endpoint | const char* | The mobile junction that is configured on ISAM instance.   |
| isSSL | const bool | true if https is used |
| clientId | const char* | The client identifier issued to the client during the registration process. |
| userCredCallback  | int (\*)(char \*\*username, char **password)  | Client callback function that will help provide username and password of the user. |
| passwordCallback | int (\*)(const char \*username, char **password) | Client callback function that will help provide password of the user `username` |
### Returns
| Type | Description |
| ---- | ----------- |
| int | _SUCCESS or _INVALID_INPUTS |

### Code
```c

int userCredCallback (char **username, char **password) {
  ...
  username = (char *) malloc (MAX_PARAM_SIZE);
  password = (char *) malloc (MAX_PARAM_SIZE);

  strcpy (username, "<username>");
  strcpy (password, "<password>");

  return _SUCCESS;
}

int passwordCallback (const char *username, char **password) {
  printf ("\nGet the password for username %s", username);
  ...
  password = (char *) malloc (MAX_PARAM_SIZE);
  strcpy (password, "<password>");

  return _SUCCESS;
}


char *hostname = "https://<ISAM hostname>";
int port = 443;
int isSSL = true;
char *clientId = "Hfdl185NjdU45DnfF";

int result = isamInitialize (
  hostname,                               // hostname
  port,                                   // port
  isSSL,                                  // isSSL
  clientId                                // clientId
  userCredCallback,                       //User Cred callback
  passwordCallback                        //Password callback
);

printf("result %d\n", result);
```
<br/>
<a href="#calls"><UP></a>


<h2 id="authWithCallbacks">authenticateToken</h2>
The user swipes the magnetic strip/RFID tag on the reader, which forwards the token value off the tag. This value is used to request for OAUth access token from ISAM. Since the callback functions are already registered with ISAM Token Authentication 'C' Library, client will call the appropriate callback function at the correct time.

### Parameters
| Name | Type | Description |
| -----| ---- | ----------- |
| scopes | const char* | (OPTIONAL) A list of comma-seperated, case-sensitive strings. i.e scope1,scope2 |
| token | const char* | The token value read off the Magnetic strip/RFID reader |
| errorDescription | char** | Buffer to receive the error description string. |

### Returns
| Type | Description |
| ---- | ----------- |
| int | Status of the call. SUCCESS or error code. |

### Code
```c
char* token = "THE_ACCESS_TOKEN";
char* errorDescription = NULL;

int result = authenticateToken(NULL,        // scope
  token,                                    // token
  &errorDescription                         // errorDescription
);

printf ("\nERROR:: %s", errorDescription);
free (errorDescription);
```
<br/>
<a href="#calls"><_UP_></a>

## Functions to be used without callbacks

##### NOTE:: To enable this feature, the TokenAuthentication Library will have to be rebuilt without the compiler flag `_CALLBACKS_DEFINED_` set.

<h2 id="initWithoutCallbacks">isamInitialize</h2>
This function is used to initialize the ISAM 'C' library. ISAM 'C' library has to be initialized before any other APIs are called.

### Parameters
| Name | Type | Description |
| -----| ---- | ----------- |
| hostname | const char* | The URL (hostname or IP address) of ISAM instance |
| port | const int | Port number on which ISAM is listening for requests. |
| isSSL | const bool | true if https is used |
| clientId | const char* | The client identifier issued to the client during the registration process. |
### Returns
| Type | Description |
| ---- | ----------- |
| int | _SUCCESS or _INVALID_INPUTS |

### Code
```c
char* hostname = "https://<ISAM hostname>";
int port = 443;
int isSSL = true;
char* clientId = "Hfdl185NjdU45DnfF";

int result = isamInitialize (
  hostname,                               // hostname
  port,                                   // port
  isSSL,                                  // isSSL
  clientId                                // clientId
);

printf("result %d\n", result);
```

<br/>
<a href="#calls">UP</a>

<h2 id="authWithoutCallbacks">authenticateToken</h2>
The user swipes the magnetic strip/RFID tag on the reader, which forwards the token value off the tag. This value is used to request for OAUth access token from ISAM.

### Parameters
| Name | Type | Description |
| -----| ---- | ----------- |
| username | const char* | (OPTIONAL) The resource owner username |
| password | const char* | (OPTIONAL) The resource owner password |
| scopes | const char* | (OPTIONAL) A list of comma-seperated, case-sensitive strings. i.e scope1,scope2 |
| token | const char* | The token value read off the Magnetic strip/RFID reader |
| actionCode | const int | Action that is to be taken in this call. |
| errorDescription | char** | Buffer to receive the error description string. |

### Returns
| Type | Description |
| ---- | ----------- |
| int | Status of the call. SUCCESS or error code. |

### Remarks
actionCode can be one of the 3 enum values: `_REQ_REFRESH_AUTH`, `_REQ_REGISTER_USER` or `_REQ_REFRESH_PASSWORD`. These values are defined in the enum `_actionCodes`.


### Code
```c
char* token = "THE_ACCESS_TOKEN";
char* errorDescription = NULL;
int actionCode = _REQ_REFRESH_AUTH;

int result = authenticateToken(NULL,        // username
  NULL,                                     // password
  NULL,                                     // scope
  token,                                    // token
  actionCode,                               // actionCode
  &errorDescription                         // errorDescription
);

printf ("\nERROR:: %s", errorDescription);
free (errorDescription);


char *password = "Passw0rd";

if (result == _REFRESH_PASSWORD) {
  actionCode = _REQ_REFRESH_PASSWORD;
  int result = authenticateToken(NULL,        // username
    password,                                 // password
    NULL,                                     // scope
    token,                                    // token
    actionCode,                               // actionCode
    &errorDescription                         // errorDescription
  );
}

char *username = "testuser1";

if (result == `_REGISTER_USER`) {
  actionCode = _REQ_REGISTER_USER;
  int result = authenticateToken(username,    // username
    password,                                 // password
    NULL,                                     // scope
    token,                                    // token
    actionCode,                               // actionCode
    &errorDescription                         // errorDescription
  );
}
```
<br/>
<a href="#calls"><_UP_></a>

## Function common to both methods

<h2 id="final">isamFinalize</h2>
After the execution of the client tasks, `isamFinalize` should be called for clean-up activities.

### Parameters
NA

### Returns
NA

### Code
```c
isamFinalize();
```
<br/>
<a href="#calls">UP</a>
