/*
 * Component Name: isamTokenAuthenticationLibrary
 * File Name: isamTokenAuthApi.c
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2018
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */
#include "isamTokenAuthApi.h"

static char *_endPointUrl;
static char *_clientId;
static bool _isClientInitialized = false;

#ifdef _CALLBACKS_DEFINED_

int (*_userCredCallback)(char **username, char **password) = NULL;
int (*_passwordCallback)(const char *username, char **password) = NULL;

#endif

/*
 * This function is used to authenticate with ISAM using tokens, like an
 * RFID tag, magnetic strip ID cards etc.
 *
 * char *scopes (in): OPTIONAL - <TODO::>
 * char *token (in): MANDATORY - The token string that is used to identify
 * 					a user.
 * char **errorDescription (in): MANDATORY - Buffer to receive the error
 * 					description string.
 *
 * int (out): Status of the call.
 * 	SUCCESS or error code.
 */
int authenticateTokenWithNoUserCreds(const char *scopes, const char *token,
		char **errorDescription) {

	int retVal = _FAILURE;
	int paramsCount = 0;

	char *result = NULL;
	char *error_desc = NULL;
	char **params = NULL;

	/*
	 * Create PARAMS
	 */
	params = getParams(&paramsCount, REFRESH_TOKEN, token, NULL, NULL);

	/*
	 * Send the token info to ISAM and check if the user is
	 * successfully authenticating or not.s
	 */
	result = get_oauth_token(_endPointUrl, _clientId, scopes, params,
			paramsCount);
	/*
	 * Check for returned JSON payload from the server and
	 * act accordingly.
	 */
	if (result != NULL) {
#ifdef DEBUG
		printf("\nTrying to get oauth token results in %s", result);
#endif

		/*
		 * Check if error is returned
		 */
		if (strstr(result, "error") != NULL) {
			/*
			 * ISAM returned error back. Check for the error_description
			 */
			error_desc = findErrorReason(result);
			if (error_desc != NULL) {
				/*
				 * Find the error reason here.
				 */
				retVal = errorCodeReturn(error_desc);
#ifdef DEBUG
				printf("\nERROR:: %s\n", *errorDescription);
#endif

				as_printf(errorDescription, "%s", error_desc);

				free(error_desc);
				error_desc = NULL;
			} else {
#ifdef DEBUG
				printf("\nERROR:: Unknown error occured.\n");
#endif
				as_printf(errorDescription, "%s", UNABLE_TO_GET_ERROR_DESC);
				retVal = _UNKNOWN_ERROR;
			}
		} else {
#ifdef DEBUG
			printf("\nUser authenticated successfully.\n");
#endif
			as_printf(errorDescription, "%s", SUCCESSFUL_AUTH);
			retVal = _SUCCESS;
		}
	} else {
#ifdef DEBUG
		printf("\nDid not receive any result from the call. Unknown error.\n");
#endif
		as_printf(errorDescription, "%s", UNKNOWN_ERROR);
		retVal = _UNKNOWN_ERROR;
	}

	/*
	 * Free result here.
	 */
	if (result != NULL) {
		free(result);
		result = NULL;
	}
	if (params != NULL) {
		freeParams(params, paramsCount);
	}
	return retVal;
}
/*
 * This function is user to authenticate against ISAM using tokens, like
 * an RFID tag, magnetic strip ID cards etc.
 *
 * char *password (in): OPTIONAL - Password of the user (can be NULL).
 * char *scopes (in): OPTIONAL - <TODO::>
 * char *token (in): MANDATORY - The token string that is used to identify
 * 					a user.
 * char **errorDescription (in): MANDATORY - Buffer to receive the error
 * 					description string.
 *
 * int (out): Status of the call.
 * 	SUCCESS or error code.
 */
int authenticateTokenWithPassword(const char* password, const char *scopes,
		const char *token, char **errorDescription) {

	int retVal = _INVALID_INPUTS;
	int paramsCount = 0;

	char *result = NULL;
	char **params = NULL;
	char *error_desc = NULL;

	if (!password || !token) {
#ifdef DEBUG
		printf("\nUsername OR Password OR Token not set.");
#endif
		retVal = _INVALID_INPUTS;
	} else {
		/*
		 * Create PARAMS
		 */
		params = getParams(&paramsCount, REFRESH_TOKEN, token, NULL, password);

		/*
		 * Send the token info to ISAM and check if the user is
		 * successfully authenticating or not.s
		 */
		result = get_oauth_token(_endPointUrl, _clientId, scopes, params,
				paramsCount);
		/*
		 * Check for returned JSON payload from the server and
		 * act accordingly.
		 */
		if (result != NULL) {
#ifdef DEBUG
			printf("\nTrying to get oauth token results in %s", result);
#endif

			/*
			 * Check if error is returned
			 */
			if (strstr(result, "error") != NULL) {
				/*
				 * ISAM returned error back. Check for the error_description
				 */
				error_desc = findErrorReason(result);
				if (error_desc != NULL) {
					as_printf(errorDescription, "%s", error_desc);

					/*
					 * Find the error reason here.
					 */
					retVal = errorCodeReturn(error_desc);
					free(error_desc);
					error_desc = NULL;
				} else {
					retVal = _UNKNOWN_ERROR;
				}
			} else {
				retVal = _SUCCESS;
			}
		} else {
			retVal = _UNKNOWN_ERROR;
		}

		/*
		 * Free result here.
		 */
		if (result != NULL) {
			free(result);
			result = NULL;
		}
		if (params != NULL) {
			freeParams(params, paramsCount);
		}
	}
	return retVal;
}
/*
 * This function is to register a user with ISAM using tokens, like an
 * RFID tag, magnetic strip ID cards etc.
 *
 * char *username (in): OPTIONAL - Username of the user (can be NULL).
 * char *password (in): OPTIONAL - Password of the user (can be NULL).
 * char *scopes (in): OPTIONAL - <TODO::>
 * char *token (in): MANDATORY - The token string that is used to identify
 * 					a user.
 * char **errorDescription (in): MANDATORY - Buffer to receive the error
 * 					description string.
 *
 * int (out): Status of the call.
 * 	SUCCESS or error code.
 */
int registerTokenWithUserCreds(const char *username, const char* password,
		const char *scopes, const char *token, char **errorDescription) {

	int retVal = _INVALID_INPUTS;
	int paramsCount = 0;

	char *result = NULL;
	char **params = NULL;
	char *error_desc = NULL;

	if (!username || !password || !token) {
#ifdef DEBUG
		printf("\nUsername OR Password OR Token not set.");
#endif
		retVal = _INVALID_INPUTS;
	} else {
		/*
		 * Create PARAMS
		 */
		params = getParams(&paramsCount, REGISTER_TOKEN, token, username,
				password);

		/*
		 * Send the token info to ISAM and check if the user is
		 * successfully authenticating or not.s
		 */
		result = get_oauth_token(_endPointUrl, _clientId, scopes, params,
				paramsCount);
		/*
		 * Check for returned JSON payload from the server and
		 * act accordingly.
		 */
		if (result != NULL) {
#ifdef DEBUG
			printf("\nTrying to get oauth token results in %s", result);
#endif

			/*
			 * Check if error is returned
			 */
			if (strstr(result, "error") != NULL) {
				/*
				 * ISAM returned error back. Check for the error_description
				 */
				error_desc = findErrorReason(result);
				if (error_desc != NULL) {
					as_printf(errorDescription, "%s", error_desc);
					/*
					 * Find the error reason here.
					 */
					retVal = errorCodeReturn(error_desc);
					free(error_desc);
					error_desc = NULL;
				} else {
					retVal = _UNKNOWN_ERROR;
				}
			} else {
				retVal = _SUCCESS;
			}
		} else {
			retVal = _UNKNOWN_ERROR;
		}

		/*
		 * Free result here.
		 */
		if (result != NULL) {
			free(result);
			result = NULL;
		}
		if (params != NULL) {
			freeParams(params, paramsCount);
		}
	}
	return retVal;
}

void freeParams(char **newParams, const int newParamsCount) {
	int i = 0;

	if (newParams) {
		for (i = 0; i < newParamsCount; i++) {
			if (newParams[i]) {
				free(newParams[i]);
				newParams[i] = NULL;
			}
		}
		free(newParams);
		newParams = NULL;
	}
}

char* findErrorReason(const char *errorResult) {
	char *errorDescription = NULL;
	char *tempString1 = NULL;
	char *tempString2 = NULL;
	char *result = NULL;
#if defined _WIN32 || _WIN64
	char *nextToken = NULL;
#endif

	if (errorResult) {
		as_printf(&tempString1, "%s", errorResult);
		if (tempString1) {
			tempString2 = tempString1;

			errorDescription = strtok(tempString1, "\",:{}");
			while (errorDescription) {
				if (strcmp(errorDescription, "error_description") == 0) {
					errorDescription = strtok(NULL, "\",:{}");
					if (errorDescription) {
						as_printf(&result, "%s", errorDescription);
					}
					break;
				}
				errorDescription = strtok(NULL, "\",:{}");
			}

			free(tempString2);
			tempString2 = NULL;
		}
	}

	return result;
}

int errorCodeReturn(char *errorDescription) {

	int retVal = _FAILURE;

	if (strcasecmp(errorDescription, USER_NOT_REGISTERED) == 0) {
		retVal = _REGISTER_USER;

	} else if (strstr(errorDescription, PASSWORD_REFRESH) != NULL) {
		retVal = _REFRESH_PASSWORD;

	} else if (strcasecmp(errorDescription, ANOTHER_CARD_ALREADY_REGISTERED)
			== 0) {
		retVal = _USER_REGISTERED;

	} else if (strcasecmp(errorDescription, GRANT_ACTIVE) == 0) {
		retVal = _SUCCESS;

	} else if (strstr(errorDescription, INVALID_ARGUMENT) != NULL) {
		retVal = _INVALID_INPUTS;

	} else if (strcasecmp(errorDescription, CARD_DISABLED) == 0) {
		retVal = _CARD_DISABLED;
	} else if (strcasecmp(errorDescription, TOKEN_LOCKED) == 0) {
		retVal = _TOKEN_LOCKED;
	} else if (strcasecmp(errorDescription, CARD_ALREADY_REGISTERED) == 0) {
		retVal = _CARD_REGISTERED;
	}
	return retVal;
}

char** getParams(int *paramsCount, int grantType, const char *token,
		const char *username, const char *password) {
	char **params = NULL;
	char *grantTypeString = NULL;
	int i = 0;

	switch (grantType) {
	case REFRESH_TOKEN:
		if (password)
			*paramsCount = 4;
		else
			*paramsCount = 3;

		as_printf(&grantTypeString, "%s", "grant_type=refresh_token");
		break;

	case REGISTER_TOKEN:
		*paramsCount = 5;
		as_printf(&grantTypeString, "%s", "grant_type=register_token");
		break;

	default:
		break;
	}

	params = (char **) malloc((*paramsCount) * sizeof(char *));

	if (params) {

		/*
		 * Copy the common parameters in the string array.
		 */
		as_printf(&params[i], "token_at=at_%s", token);
		as_printf(&params[++i], "%s", grantTypeString);

		switch (grantType) {
		case REFRESH_TOKEN:
			as_printf(&params[++i], "refresh_token=rt_%s", token);

			if (password) {
				as_printf(&params[++i], "password=%s", password);
			}
			break;

		case REGISTER_TOKEN:
			//as_printf(&params[++i], "refresh_token=rt_%s", token);
			as_printf(&params[++i], "token_rt=rt_%s", token);
			as_printf(&params[++i], "username=%s", username);
			as_printf(&params[++i], "password=%s", password);

			break;
		default:
			break;
		}
	}

	if (grantTypeString) {
		free(grantTypeString);
		grantTypeString = NULL;
	}

	return params;
}

#ifndef _CALLBACKS_DEFINED_
/*
 * This function is called by the client to initialize the 'C'client library.
 * This function need to be called before any other method is called.
 *
 * char *hostname (in): URL (or hostname/IP address of ISAM instance).
 * int port (in): Port number on which ISAM is listening to the incoming requests.
 * char *endpoint (in): Mobile junction configured on ISAM.
 * bool isSSL (in): Is the connection un/secured (http OR https)
 * char *clientId (in): Client Id configured for this client lib on ISAM.
 *
 * int (out): Status of the call.
 * 	SUCCESS or error code.
 */
int isamInitialize(const char *hostname, const int port, const char *endpoint,
		const bool isSSL, const char *clientId) {

	int retVal = _FAILURE;

	if (!_isClientInitialized) {
		if (!hostname || !endpoint || !clientId) {
			retVal = _INVALID_INPUTS;
			_isClientInitialized = false;
		} else {
			_endPointUrl = getEndpointUrl(hostname, port, endpoint, isSSL);
			if (_endPointUrl) {
				as_printf(&_clientId, "%s", clientId);
				retVal = _SUCCESS;
				_isClientInitialized = true;
			}
		}
	}
	return retVal;
}

/*
 * This function is called by the client to enable a user to authenticate
 * against ISAM using tokens, like an RFID tag, magnetic strip ID cards etc.
 *
 * char *username (in): OPTIONAL - Username of the user (can be NULL).
 * char *password (in): OPTIONAL - Password of the user (can be NULL).
 * char *scopes (in): OPTIONAL - <TODO::>
 * char *token (in): MANDATORY - The token string that is used to identify
 * 					a user.
 * int actionCode (in): MANDATORY - Action to be taken (One of the following:
 * _REQ_REFRESH_AUTH, _REQ_REGISTER_USER OR _REQ_REFRESH_PASSWORD.
 *
 * char **errorDescription (in): MANDATORY - Buffer to receive the error
 * 					description string.
 *
 * int (out): Status of the call.
 * 	_SUCCESS or error code.
 */
int authenticateToken(const char *username, const char *password,
		const char *scopes, const char *token, const int actionCode,
		char **errorDescription) {

	int retVal = _FAILURE;

	if (_isClientInitialized) {
		if (token == NULL) {
#ifdef DEBUG
			printf("\nToken is not passed. It is a required parameter.");
#endif
			retVal = _INVALID_INPUTS;
			as_printf(errorDescription, "%s", INVALID_INPUT);
		} else {

			switch (actionCode) {
			case _SUCCESS:
				retVal = authenticateTokenWithNoUserCreds(scopes, token,
						errorDescription);
				break;
			case _REGISTER_USER:
				retVal = registerTokenWithUserCreds(username, password, scopes,
						token, errorDescription);
				break;
			case _REFRESH_PASSWORD:
				retVal = authenticateTokenWithPassword(password, scopes, token,
						errorDescription);
				break;
			default:
				break;
			}
		}
	} else {
		retVal = _CLIENT_NOT_INITIALIZED;
	}
	return retVal;
}

#else

int isamInitialize(const char *hostname, const int port,
		const char *endpoint, const bool isSSL, const char *clientId,
		int (*userCredCallback)(char **username, char **password),
		int (*passwordCallback)(const char *username, char **password)) {

	int retVal = _FAILURE;

	if (!_isClientInitialized) {
		if (!hostname || !endpoint || !clientId || !userCredCallback ||
				!passwordCallback) {
			retVal = _INVALID_INPUTS;
			_isClientInitialized = false;
		} else {
			_endPointUrl = getEndpointUrl(hostname, port, endpoint, isSSL);
			if (_endPointUrl) {
				as_printf(&_clientId, "%s", clientId);

				_userCredCallback = userCredCallback;
				_passwordCallback = passwordCallback;

				retVal = _SUCCESS;
				_isClientInitialized = true;
			}
		}
	}
	return retVal;
}

int authenticateToken(const char *scopes, const char *token,
		char **errorDescription) {

	int retVal = _SUCCESS;
	char *username = NULL;
	char *password = NULL;

	if (_isClientInitialized) {
		if (token) {
			do {
				switch (retVal) {

					case _SUCCESS:
					retVal = authenticateTokenWithNoUserCreds(scopes, token,
							errorDescription);
					break;

					case _REFRESH_PASSWORD:
					username = getUserAssociatedWithToken(*errorDescription);
					retVal = _passwordCallback(username, &password);
					if (retVal == _SUCCESS && password) {
						retVal = authenticateTokenWithPassword(password, scopes,
								token, errorDescription);
					} else {
						/*
						 * Callback function returned back with _FAILURE or
						 * password is NULL. Do the cleanup.
						 */
						retVal = _INVALID_INPUTS;
					}
					if (username) {
						free (username);
						username = NULL;
					}
					if (password) {
						free (password);
						password = NULL;
					}
					break;

					case _REGISTER_USER:
					retVal = _userCredCallback(&username, &password);
					if (retVal == _SUCCESS && username && password) {
						retVal = registerTokenWithUserCreds(username, password,
								scopes, token, errorDescription);

					} else {
						/*
						 * Either the callback returned back with a _FAILURE or
						 * username or password is NULL. Do cleanup and return
						 * back to the main callee.
						 */
						retVal = _INVALID_INPUTS;
					}
					/*
					 * Free username AND/OR password if allocated.
					 */
					if (username) {
						free (username);
						username = NULL;
					}
					if (password) {
						free (password);
						password = NULL;
					}
					break;

					default:
					retVal = _CANNOT_CONTINUE;
					break;
				}
			}while ((retVal >= _REGISTER_USER) && (retVal <= _INVALID_INPUTS));
		} else {
			retVal = _INVALID_INPUTS;
			as_printf (errorDescription, "%s", INVALID_INPUT);
		}

		/*
		 * if retVal = _SUCCESS, change the errorDescription to "User logged in
		 * successfully".
		 */
		if (retVal == _SUCCESS) {
			if (*errorDescription) {
				free (*errorDescription);
				*errorDescription = NULL;
			}
			as_printf(errorDescription, "%s", SUCCESSFUL_AUTH);
		}
	} else {
		retVal = _CLIENT_NOT_INITIALIZED;
		as_printf(errorDescription, "%s", CLIENT_NOT_INITIALIZED);
	}

	return retVal;
}
#endif

/*
 * This function is called by the client finalize the 'C'client library.
 * This should be called whenever to complete the execution of the client
 * application to cleanup the resources.
 */
void isamFinalize() {
	if (_isClientInitialized) {
		if (_endPointUrl) {
			free(_endPointUrl);
			_endPointUrl = NULL;
		}
		if (_clientId) {
			free(_clientId);
			_clientId = NULL;
		}
	}
}
