/*
 * Component Name: isamTokenAuthenticationLibrary
 * File Name: isamTokenAutRestApi.c
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2018
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */
#include "isamTokenAuthRestApi.h"

/*
 Helper Functions
 */

/*
 * Creates a JSON formatted error result.
 *
 * error: A pointer to the error.
 * description: A pointer to the error description.
 * returns: A pointer to the error message.
 */
char* json_error_message(const char* error, const char* description) {
	char* result = NULL;
	as_printf(&result, "{ \"error\":\"%s\",\"error_description\":\"%s\" }",
			error, description);

	return result;
}

/*
 * Validates a function argument.

 * name: A pointer to the name of the argument.
 * value: A pointer to the value of the argument.
 * result: (OUT) A pointer to the result error string if false is returned.
 * returns: true if validation was successful, otherwise false.
 */
bool validate_argument(const char* name, const char* value, char** result) {
	bool retVal = true;

	// Check to make sure the value is not NULL or empty.
	if (value && !value[0]) {
		// asprintf automatically allocates the memory it needs.
		char* description = NULL;
		char* jsonError = NULL;

		as_printf(&description, "'%s' was NULL or empty.", name);
		jsonError = json_error_message("invalid_argument", description);
		as_printf(result, "%s", jsonError);

		free(description);
		description = NULL;

		free(jsonError);
		jsonError = NULL;

		retVal = false;
	}
	return retVal;
}

/*
 Creates a string to be added to the request body.

 * value: A pointer to the array of elements.
 * count: A integer representing the number of elements in value[].
 * returns: A pointer to the rquest body value, otherwise NULL.
 */
char* create_request_body_value(char* value[], int count) {

	int index = 0;
	char* buffer = NULL;

	if (value && count > 0) {
		// Allocate memory.
		//buffer = (char*) calloc(MAX_PARAM_SIZE, sizeof(char));
		buffer = (char*) malloc(MAX_PARAM_SIZE * sizeof(char));
		buffer[0] = '\0';

		for (index = 0; index < count; index++) {
			// Truncate the array values if it can't be added to the buffer.
			if ((strlen(buffer) + strlen(value[index])) < (MAX_PARAM_SIZE - 1)) {
				strncat(buffer, "&", 1);
				strncat(buffer, value[index], strlen(value[index]));
			}
		}
	}
	return buffer;
}

/*
 * cURL callback for writing received data.
 *
 * ptr: A pointer to the response data.
 * size: Size of the ptr data.
 * nmemb: The multiplied size of size.
 * userdata: Set with the CURLOPT_WRITEDATA option.
 * return: Size of the response.
 * remarks: Refer to the https://curl.haxx.se/libcurl/c/CURLOPT_WRITEFUNCTION.html for more information on the function requirements.
 */
size_t curl_request_callback(char *ptr, size_t size, size_t nmemb,
		void *userdata) {
	size_t result = size * nmemb;

	struct CALLBACK_DATA *data = (struct CALLBACK_DATA*) userdata;
	data->buffer = realloc(data->buffer, data->size + result + 1);

	if (data->buffer == NULL) {
		// Out of memory!
		printf("not enough memory (realloc returned NULL)\n");
		return 0;
	}

	memcpy(&(data->buffer[data->size]), ptr, result);
	data->size += result;
	data->buffer[data->size] = 0;

	return result;
}

/*
 * Executes a generic HTTP request via the libcurl library.
 *
 * url: A pointer to the target URL.
 * params: A pointer to the parameters to be submitted with the request.
 */
char* curl_execute_request(char* url, char* params) {
	CURLcode response;      // The cURL response.
	CURL* curl = NULL;             // The handle to the cURL reference.
	char* result = NULL;    // The returning result.

	struct CALLBACK_DATA data; // Create the structure to hold the callback data.
	struct curl_slist *headers = NULL;

	// Initial the curl handle.
	curl = curl_easy_init();
	if (!curl) {
		return json_error_message("libcurl",
				"Unable to establish handle from curl_easy_init().");
	}

//	struct CALLBACK_DATA data; // Create the structure to hold the callback data.
	data.buffer = malloc(1); // Will be expanded when response is handled in callback.
	data.size = 0;

//	struct curl_slist *headers = NULL;
	headers = curl_slist_append(headers, "Accept: application/json");
	headers = curl_slist_append(headers, "charsets: utf-8");

	curl_easy_setopt(curl, CURLOPT_URL, url);           // Set the endpoint url.
	curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers); // Add the headers to the request.
	curl_easy_setopt(curl, CURLOPT_POST, 1L);            // We are doing a POST.
	curl_easy_setopt(curl, CURLOPT_POSTFIELDS, params); // Assign post parameters.
	curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, strlen(params));
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_request_callback); // Assign the request callback function.
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void* ) &data);

	/*
	 * TODO:: The 3 lines below need to be removed. They should not go in
	 * the final library build.
	 */
	curl_easy_setopt(curl, CURLOPT_VERBOSE, CURL_VERBOSE);	// Debug
	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, CURL_VERIFY_PEER); //Do not verify certs
	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, CURL_VERIFY_HOST); //Do not verify certs

	/*
	 * TODO:: The 3 lines above need to be removed. They should not go in
	 * the final library build.
	 */

	response = curl_easy_perform(curl);                  // Process the request.

	if (response != CURLE_OK)           // Request fails, write a JSON response.
	{
		result = json_error_message("libcurl", curl_easy_strerror(response));
	} else {
		as_printf(&result, "%s", data.buffer);
	}

	// Clean up.
	curl_slist_free_all(headers);
	headers = NULL;

	curl_easy_cleanup(curl);
	curl = NULL;

	free(data.buffer);
	data.buffer = NULL;

	return result;
}

/*
 * Checks if grant_type is passed as a part of the params array.
 *
 * params: pointer to the string array
 * paramsCount: Number of strings present in the strings array
 * returns: true if grant_type is present, else false.
 */
bool is_grant_type_present(char *params[], int paramsCount, char **error) {
	bool retVal = false;
	int i = 0;

	if (params) {
		for (; i < paramsCount; i++) {
			if (strstr(params[i], "grant_type") == 0) {
				retVal = true;
				break;
			} else {
				char* description = NULL;
				as_printf(&description, "grant_type is not set.");
				as_printf(error, "%s",
						json_error_message("invalid_argument", description));

				free(description);
			}
		}
	}

	return retVal;
}

/* 
 Header Functions
 */

char* get_oauth_token_resource_owner(char* endpointUrl, const char* clientId,
		const char* clientSecret, const char* username, const char* password,
		const char* scopes, char* params[], const int paramsCount) {
	// Validate the arguments.
	char* error = NULL;
	char* buffer = NULL;

	if (!validate_argument("endpointUrl", endpointUrl, &error)) {
		return error;
	}

	if (!validate_argument("clientId", clientId, &error)) {
		return error;
	}

	if (!validate_argument("username", username, &error)) {
		return error;
	}

	if (!validate_argument("password", password, &error)) {
		return error;
	}

	// Create the initial body parameters.
//	char* buffer = NULL;
	as_printf(&buffer, "client_id=%s&username=%s&password=%s", clientId,
			username, password);

	bool hasGrantType = false;

	if (clientSecret && strlen(clientSecret) > 0) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(clientSecret + 15)) * sizeof(char));
		strcat(buffer, "&client_secret=");
		strcat(buffer, clientSecret);
	}

	if (scopes && strlen(scopes) > 0) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(scopes + 7)) * sizeof(char));
		strcat(buffer, "&scope=");
		strcat(buffer, scopes);
	}

	if (params) {
		char* result = create_request_body_value(params, paramsCount);

		// Check if the result has a grant_type.
		hasGrantType = (strstr(result, "grant_type") != NULL);

		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(result)) * sizeof(char));
		strcat(buffer, result);
		free(result);
	}

	// No grant_type supplied in the params, add the default.
	if (!hasGrantType) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen("&grant_type=password"))
						* sizeof(char));
		strcat(buffer, "&grant_type=password");
	}

	return curl_execute_request(endpointUrl, buffer);
}

char* get_oauth_token_auth_code(char* endpointUrl, const char* clientId,
		const char* clientSecret, const char* code, const char* scopes,
		char* params[], const int paramsCount) {
	// Validate the arguments.
	char* error = NULL;
	char* buffer = NULL;

	if (!validate_argument("endpointUrl", endpointUrl, &error)) {
		return error;
	}

	if (!validate_argument("clientId", clientId, &error)) {
		return error;
	}

	if (!validate_argument("code", code, &error)) {
		return error;
	}

	// Create the initial body parameters.
//	char* buffer = NULL;
	as_printf(&buffer, "grant_type=authorization_code&client_id=%s&code=%s",
			clientId, code);

	if (clientSecret && strlen(clientSecret) > 0) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(clientSecret + 15)) * sizeof(char));
		strcat(buffer, "&client_secret=");
		strcat(buffer, clientSecret);
	}

	if (scopes && strlen(scopes) > 0) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(scopes + 7)) * sizeof(char));
		strcat(buffer, "&scope=");
		strcat(buffer, scopes);
	}

	if (params) {
		char* result = create_request_body_value(params, paramsCount);

		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(result)) * sizeof(char));
		strcat(buffer, result);
		free(result);
	}

	return curl_execute_request(endpointUrl, buffer);
}

char* get_oauth_refresh_token(char* endpointUrl, const char* clientId,
		const char* refreshToken, const char* scopes, char* params[],
		const int paramsCount) {
	// Validate the arguments.
	char* error = NULL;
	char* buffer = NULL;

	if (!validate_argument("endpointUrl", endpointUrl, &error)) {
		return error;
	}

	if (!validate_argument("clientId", clientId, &error)) {
		return error;
	}

	if (!validate_argument("params", *params, &error)) {
		return error;
	}

	// Create the initial body parameters.
	as_printf(&buffer, "client_id=%s&grant_type=refresh_token&refresh_token=%s",
			clientId, refreshToken);

	if (scopes && strlen(scopes) > 0) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(scopes + 7)) * sizeof(char));
		strcat(buffer, "&scope=");
		strcat(buffer, scopes);
	}

	if (params) {
		char* result = create_request_body_value(params, paramsCount);

		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(result)) * sizeof(char));
		strcat(buffer, result);
		free(result);
	}

	return curl_execute_request(endpointUrl, buffer);
}

/*
 * Generic oAuth API for any kind of oAuth call.
 */
char* get_oauth_token(char* endpointUrl, const char* clientId,
		const char* scopes, char* params[], const int paramsCount) {
	// Validate the arguments.
	char* error = NULL;
	char* buffer = NULL;
	char* result = NULL;
	char* restApiResult = NULL;

	if (!validate_argument("endpointUrl", endpointUrl, &error)) {
		return error;
	}

	if (!validate_argument("clientId", clientId, &error)) {
		return error;
	}

	if (params) {
		if (!is_grant_type_present(params, paramsCount, &error)) {
			return error;
		}
	}

	// Create the initial body parameters.
	as_printf(&buffer, "client_id=%s", clientId);

	//TODO:: Pranab:: Check the implementation on the original code that
	//came from Craig.
	if (scopes && strlen(scopes) > 0) {
		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(scopes) + 7) * sizeof(char));
		strcat(buffer, "&scope=");
		strcat(buffer, scopes);
	}

	if (params) {
		result = create_request_body_value(params, paramsCount);

		buffer = (char*) realloc(buffer,
				(strlen(buffer) + strlen(result) + 1) * sizeof(char));
		strncat(buffer, result, strlen(result));
	}

	restApiResult = curl_execute_request(endpointUrl, buffer);
	if (result) {
		free(result);
		result = NULL;
	}
	free(buffer);
	buffer = NULL;

	return restApiResult;
}
