/*
 * Component Name: isamTokenAuthenticationLibrary
 * File Name: isamTokenAuthRestApi.h
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2018
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */

#ifndef _ISAM_TOKEN_AUTH_REST_API_H_
#define _ISAM_TOKEN_AUTH_REST_API_H_

#if defined _WIN32 || _WIN64
#include "curl.h"
#else
#include <curl/curl.h>
#endif

#include "isamTokenAuthUtils.h"
#include "isamTokenAuthExterns.h"


// Constants
#ifndef NULL
#define NULL 0
#endif

#ifdef DEBUG
#define CURL_VERBOSE 1L
#define CURL_VERIFY_PEER 0
#define CURL_VERIFY_HOST 0
#else
#define CURL_VERBOSE 0
#define CURL_VERIFY_PEER 1L
#define CURL_VERIFY_HOST 1L
#endif

/*
 Provides a count of the elements in char* value[].
 */
#ifndef PARAM_COUNT
#define PARAM_COUNT(value) (sizeof(value) / sizeof(char*))
#endif


/*
 Structure to contain the cURL callback data.
 */
struct CALLBACK_DATA {
	/*
	 A pointer to a char array.
	 */
	char* buffer;

	/*
	 The size of the response data.*/
	size_t size;
};

/*
 * The resource owner password credentials (i.e., username and password) can be used directly as an authorization grant to obtain an access token.
 *
 * endpointUrl: The URL used by the client to exchange an authorization grant for an access token.
 * clientId: The client identifier issued to the client during the registration process.
 * clientSecret: (OPTIONAL) The client secret.  The client MAY omit the parameter if the client secret is an empty string.
 * username:  The resource owner username.
 * password:  The resource owner password.
 * scopes: (OPTIONAL) A list of comma-seperated, case-sensitive strings. i.e scope1,scope2
 * params: (OPTIONAL) A comma-seperated list of parameters in a form urlencoded format. i.e key1=value1,key2=value2
 * paramsCount: The number of elements in params.  No validation of the paramsCount is applied to the number of elements in params.  Use ARRAY_COUNT(params) to get count.
 * returns: The JSON representing the response form the authorization server.
 * remarks: The grant_type by default is password.  To define a different grant_type, supply the desired grant type in the `params` argument i.e grant_type=my_custom_type
 */
char* get_oauth_token_resource_owner(char* endpointUrl,
		const char* clientId, const char* clientSecret, const char* username,
		const char* password, const char* scopes, char* params[],
		const int paramsCount);

/*
 * The authorization code is obtained by using an authorization server as an intermediary between the client and resource owner.
 *
 * endpointUrl: The URL used by the client to exchange an authorization grant for an access token.
 * clientId: The client identifier issued to the client during the registration process.
 * clientSecret: The client secret.  The client MAY omit the parameter if the client secret is an empty string.
 * code:  The authorization code generated by the authorization server.
 * scopes: (OPTIONAL) A list of comma-seperated, case-sensitive strings. i.e scope1,scope2
 * params: (OPTIONAL) A comma-seperated list of parameters in a form urlencoded format. i.e key1=value1,key2=value2
 * paramsCount: The number of elements in params.  No validation of the paramsCount is applied to the number of elements in params.  Use ARRAY_COUNT(params) to get count.
 * returns: The JSON representing the response form the authorization server.
 */
char* get_oauth_token_auth_code(char* endpointUrl, const char* clientId,
		const char* clientSecret, const char* code, const char* scopes,
		char* params[], const int paramsCount);

/*
 * Refresh tokens are issued to the client by the authorization server and are used to obtain a new access token when the current access token becomes invalid or expires.
 *
 * endpointUrl: The URL used by the client to exchange an authorization grant for an access token.
 * clientId: The client identifier issued to the client during the registration process.
 * refreshToken: The refresh token issued to the client.
 * scopes: (OPTIONAL) A list of comma-seperated, case-sensitive strings. i.e scope1,scope2
 * params: (OPTIONAL) A comma-seperated list of parameters in a form urlencoded format. i.e key1=value1,key2=value2
 * paramsCount: The number of elements in params.  No validation of the paramsCount is applied to the number of elements in params.  Use ARRAY_COUNT(params) to get count.
 * returns: The JSON representing the response form the authorization server.
 */
char* get_oauth_refresh_token(char* endpointUrl, const char* clientId,
		const char* refreshToken, const char* scopes, char* params[],
		const int paramsCount);

char* get_oauth_token(char* endpointUrl, const char* clientId,
		const char* scopes, char* params[], const int paramsCount);
#endif /* _ISAM_TOKEN_AUTH_REST_API_H_ */

