/*
 * Component Name: isamTokenAuthenticationLibrary
 * File Name: isamTokenAuthUtils.c
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2018
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */
#include "isamTokenAuthUtils.h"

const char *tokenPath = "/sps/oauth/oauth20/token";

/*
 * This function utilizes the hostname, port and endpoint information and
 * creates a final endpoint url that can be used for ISAM oauth library
 * for requesting information off ISAM.
 *
 * char *hostname(in)
 * int port (in)
 * char *endpoint (in)
 *
 * char *result_endpoint (out)/NULL
 */
char* create_full_endpoint_url(const char *hostname, const int port,
		const char *endpoint) {
	char *result_endpoint = NULL;

	if (hostname != NULL && endpoint != NULL) {

		/*
		 * <-------hostname------>:<port>/<ep>/<-tokenPath->
		 * http(s)://ISAM_HOSTNAME:PORT/endpoint/tokenPath
		 */
		as_printf(&result_endpoint, "%s:%d%s%s", hostname, port, endpoint,
				tokenPath);

#ifdef DEBUG
		printf("\nFinal endpoint URL is %s.\n", result_endpoint);
#endif

	}
	return result_endpoint;
}

/*
 * Utility function that is used to check if the hostname has the preceeding
 * http:// OR https://. If not, it returns false.
 */
int check_url(const char *hostname) {
	int result = _FAILURE;
	if ((strstr(hostname, "http://")) || strstr(hostname, "https://")) {
		result = _SUCCESS;
	}

	return result;
}

/*
 * DEBUG function for printing all the params that are passed by the user.
 */
void printParams(const char **value, const int count) {

	if (value && count > 0) {
		// Allocate memory.
		char* buffer = (char*) calloc(MAX_PARAM_SIZE, sizeof(char));

		for (int index = 0; index < count; index++) {
			// Truncate the array values if it can't be added to the buffer.
			if ((strlen(buffer) + strlen(value[index])) < MAX_PARAM_SIZE - 1) {
				strcat(buffer, "&");
				strcat(buffer, value[index]);
			}
		}
	}
}
/*
 * POSIX compliant implementation for calculating the string length with
 * variable parameters.
 */
int calculateStringSize(const char *fmt, va_list args) {
	int stringLen = 0;

	va_list ap;

	va_copy(ap, args);

	stringLen = vsnprintf(NULL, 0, fmt, ap);

	va_end(ap);
	return stringLen;
}

/*
 * POSIX compliant implementation of asprintf() function.
 */
int as_printf(char **dest, const char *fmt, ...) {
	int stringLen = 0;
	int result = 0;

	va_list ap;
	va_start(ap, fmt);

	stringLen = calculateStringSize(fmt, ap);

	if (stringLen > 0) {
		*dest = (char *) malloc((stringLen + 1) * sizeof(char));
		if (*dest != NULL) {
			result = vsnprintf(*dest, (stringLen + 1), fmt, ap);
			if (result <= 0 || result > stringLen) {
				free(*dest);
				*dest = NULL;
				result = -1;
			}
		}
	}
	va_end(ap);

	return result;
}

char* removeSingleQuotes(const char *username) {
	char *tempString = NULL;
	char *newString = NULL;

	if (username) {
		tempString = strdup(username);

		/*
		 * The username string is coming in the format "'username'" and we
		 * need the final string in "username" format. Size of the final
		 * string will be 2 less characters than the initial string,
		 * excluding the NULL character. So
		 * (final string length = initial string length - 2 quotes + 1 ending
		 * null character)
		 */
		newString = (char*) malloc(strlen(tempString) - 1);
		if (newString) {
			/*
			 * Since the initial string is "'username'", starting index of
			 * the source should be stringPtr++ and size should be 2 less
			 * than initial string length to remove 2 '.
			 */
			strncpy(newString, (tempString + 1), (strlen(tempString) - 2));
#ifdef DEBUG
			printf("\nUsername is %s", newString);
#endif
		}

		free(tempString);
		tempString = NULL;
	}
	return newString;
}

char* getUserAssociatedWithToken(const char *errorDescription) {
	char *newUsername = NULL;
	char *username = NULL;

	if (errorDescription) {
		newUsername = strchr(errorDescription, '\'');
		if (newUsername) {
			username = removeSingleQuotes(newUsername);
		}
	}
	return username;
}
